@@ -353,7 +353,9 @@ gst_aasink_chain (GstPad *pad, GstBuffer *buf)
   GST_DEBUG (0,"videosink: clock wait: %llu", GST_BUFFER_TIMESTAMP(buf));
 
   if (aasink->clock) {
-    gst_element_clock_wait (GST_ELEMENT (aasink), aasink->clock, GST_BUFFER_TIMESTAMP(buf), NULL);
+    GstClockID id = gst_clock_new_single_shot_id (aasink->clock, GST_BUFFER_TIMESTAMP(buf));
+    gst_element_clock_wait (GST_ELEMENT (aasink), id, NULL);
+    gst_clock_id_free (id);
   }
 
   aa_render (aasink->context, &aasink->ascii_parms, 
