@@ -149,10 +149,10 @@ static gboolean 	gst_mad_src_event 	(GstPad *pad, GstEvent *event);
 static const GstFormat* gst_mad_get_formats 	(GstPad *pad);
 static const GstEventMask* 
 			gst_mad_get_event_masks (GstPad *pad);
-static const GstPadQueryType* 
+static const GstQueryType* 
 			gst_mad_get_query_types (GstPad *pad);
 
-static gboolean 	gst_mad_src_query 	(GstPad *pad, GstPadQueryType type,
+static gboolean 	gst_mad_src_query 	(GstPad *pad, GstQueryType type,
 		  				 GstFormat *format, gint64 *value);
 static gboolean 	gst_mad_convert_sink 	(GstPad *pad, GstFormat src_format, gint64 src_value, 
 		     				 GstFormat *dest_format, gint64 *dest_value);
@@ -473,19 +473,19 @@ gst_mad_convert_src (GstPad *pad, GstFormat src_format, gint64 src_value,
   return res;
 }
 
-static const GstPadQueryType*
+static const GstQueryType*
 gst_mad_get_query_types (GstPad *pad)
 {
-  static const GstPadQueryType gst_mad_src_query_types[] = {
-    GST_PAD_QUERY_TOTAL,
-    GST_PAD_QUERY_POSITION,
+  static const GstQueryType gst_mad_src_query_types[] = {
+    GST_QUERY_TOTAL,
+    GST_QUERY_POSITION,
     0
   };
   return gst_mad_src_query_types;
 }
 
 static gboolean
-gst_mad_src_query (GstPad *pad, GstPadQueryType type,
+gst_mad_src_query (GstPad *pad, GstQueryType type,
 		   GstFormat *format, gint64 *value)
 {
   gboolean res = TRUE;
@@ -494,7 +494,7 @@ gst_mad_src_query (GstPad *pad, GstPadQueryType type,
   mad = GST_MAD (gst_pad_get_parent (pad));
 
   switch (type) {
-    case GST_PAD_QUERY_TOTAL:
+    case GST_QUERY_TOTAL:
     {
       switch (*format) {
 	case GST_FORMAT_DEFAULT:
@@ -516,7 +516,7 @@ gst_mad_src_query (GstPad *pad, GstPadQueryType type,
 	    GstFormat peer_format = *peer_formats;
 
 	    /* do the probe */
-            if (gst_pad_query (GST_PAD_PEER (mad->sinkpad), GST_PAD_QUERY_TOTAL,
+            if (gst_pad_query (GST_PAD_PEER (mad->sinkpad), GST_QUERY_TOTAL,
 			       &peer_format, &peer_value)) 
 	    {
               GstFormat conv_format;
@@ -540,7 +540,7 @@ gst_mad_src_query (GstPad *pad, GstPadQueryType type,
       }
       break;
     }
-    case GST_PAD_QUERY_POSITION:
+    case GST_QUERY_POSITION:
       switch (*format) {
 	case GST_FORMAT_DEFAULT:
           *format = GST_FORMAT_TIME;
@@ -931,6 +931,68 @@ fail:
   return caps;
 }
 
+static void
+gst_mad_handle_event (GstPad *pad, GstBuffer *buffer)
+{
+  GstEvent *event = GST_EVENT (buffer);
+  GstMad *mad = GST_MAD (gst_pad_get_parent (pad));
+
+  switch (GST_EVENT_TYPE (event))
+  {
+    case GST_EVENT_DISCONTINUOUS:
+    {
+      gint n = GST_EVENT_DISCONT_OFFSET_LEN (event);
+      gint i;
+
+      for (i = 0; i < n; i++)
+      {
+	const GstFormat *formats;
+	formats = gst_pad_get_formats (pad);
+
+        if (gst_formats_contains (formats, GST_EVENT_DISCONT_OFFSET(event, i).format))
+        {
+          gint64 value = GST_EVENT_DISCONT_OFFSET (event, i).value;
+          gint64 time;
+          GstFormat format;
+          GstEvent *discont;
+
+          /* see how long the input bytes take */
+          format = GST_FORMAT_TIME;
+          if (!gst_pad_convert (pad,
+		                GST_EVENT_DISCONT_OFFSET (event, i).format,
+				value, &format, &time))
+          {
+            time = 0;
+          }
+
+          /* for now, this is the best we can do. Let's hope a real timestamp
+           * arrives with the next buffer */
+          mad->base_time = time;
+
+          gst_event_unref (event);
+
+          if (GST_PAD_IS_USABLE (mad->srcpad))
+	  {
+            discont = gst_event_new_discontinuous (FALSE, GST_FORMAT_TIME,
+			                           time, NULL);
+            gst_pad_push (mad->srcpad, GST_BUFFER (discont));
+          }
+          break;
+        }
+      }
+      mad->total_samples = 0;
+      mad->tempsize = 0;
+      /* we don't need to restart when we get here */
+      mad->restart = FALSE;
+      break;
+    }
+    default:
+      gst_pad_event_default (pad, event);
+      break;
+  }
+  return;
+}
+
 static void
 gst_mad_chain (GstPad *pad, GstBuffer *buffer)
 {
@@ -940,59 +1002,14 @@ gst_mad_chain (GstPad *pad, GstBuffer *buffer)
   gboolean new_pts = FALSE;
 
   mad = GST_MAD (gst_pad_get_parent (pad));
+  g_return_if_fail (GST_IS_MAD (mad));
 
-  if (GST_IS_EVENT (buffer)) {
-    GstEvent *event = GST_EVENT (buffer);
-
-    switch (GST_EVENT_TYPE (event)) {
-      case GST_EVENT_DISCONTINUOUS:
-      {
-	gint n = GST_EVENT_DISCONT_OFFSET_LEN (event);
-	gint i;
-
-	for (i=0; i<n; i++) {
-	  if (gst_pad_handles_format (pad, GST_EVENT_DISCONT_OFFSET(event,i).format))
-	  {
-            gint64 value = GST_EVENT_DISCONT_OFFSET (event, i).value;
-	    gint64 time;
-	    GstFormat format;
-            GstEvent *discont;
-
-	    /* see how long the input bytes take */
-	    format = GST_FORMAT_TIME;
-	    if (!gst_pad_convert (pad,
-			GST_EVENT_DISCONT_OFFSET (event, i).format, value,
-			&format, &time))
-	    {
-	      time = 0;
-	    }
-
-	    /* for now, this is the best we can do. Let's hope a real timestamp
-	     * arrives with the next buffer */
-	    mad->base_time = time;
-
-            gst_event_unref (event);
-
-	    if (GST_PAD_IS_USABLE (mad->srcpad)) {
-	      discont = gst_event_new_discontinuous (FALSE, GST_FORMAT_TIME, time, NULL);
-              gst_pad_push (mad->srcpad, GST_BUFFER (discont));
-	    }
-	    break;
-	  }
-	}
-        mad->total_samples = 0;
-        mad->tempsize = 0;
-	/* we don't need to restart when we get here */
-        mad->restart = FALSE;
-	break;
-      }
-      default:
-	gst_pad_event_default (pad, event);
-	break;
-    }
+  /* handle events */
+  if (GST_IS_EVENT (buffer))
+  {
+    gst_mad_handle_event (pad, buffer);
     return;
   }
-
   if (GST_BUFFER_TIMESTAMP (buffer) != -1) {
     /* if there is nothing queued (partial buffer), we prepare to set the
      * timestamp on the next buffer */
@@ -1011,13 +1028,14 @@ gst_mad_chain (GstPad *pad, GstBuffer *buffer)
   data = GST_BUFFER_DATA (buffer);
   size = GST_BUFFER_SIZE (buffer);
 
-  while (size > 0) {
+  /* process the incoming buffer in chunks of maximum MDLEN bytes */
+  while (size > 0)
+  {
     gint tocopy;
     guchar *mad_input_buffer;
 
-    /* cut the buffer in MDLEN pieces */
     tocopy = MIN (MAD_BUFFER_MDLEN, size);
-	  
+
     memcpy (mad->tempbuffer + mad->tempsize, data, tocopy);
     mad->tempsize += tocopy;
 
@@ -1039,31 +1057,34 @@ gst_mad_chain (GstPad *pad, GstBuffer *buffer)
 	/* not enough data, need to wait for next buffer? */
 	if (mad->stream.error == MAD_ERROR_BUFLEN) {
 	  break;
-	} 
+	}
         if (!MAD_RECOVERABLE (mad->stream.error)) {
           gst_element_error (GST_ELEMENT (mad), "fatal error decoding stream");
           return;
         }
 	else if (mad->stream.error == MAD_ERROR_LOSTSYNC) {
+	  /* lost sync, force a resync */
 	  signed long tagsize;
 
 	  tagsize = id3_tag_query (mad->stream.this_frame,
 	                          mad->stream.bufend - mad->stream.this_frame);
 
 	  if (tagsize > mad->tempsize) {
-            GST_INFO (GST_CAT_PLUGIN_INFO, "mad: got partial id3 tag in buffer, skipping");
+            GST_INFO (GST_CAT_PLUGIN_INFO,
+		      "mad: got partial id3 tag in buffer, skipping");
 	  }
 	  else if (tagsize > 0) {
 	    struct id3_tag *tag;
 	    id3_byte_t const *data;
 
-            GST_INFO (GST_CAT_PLUGIN_INFO, "mad: got ID3 tag size %ld", tagsize);
+            GST_INFO (GST_CAT_PLUGIN_INFO,
+		      "mad: got ID3 tag size %ld", tagsize);
 
 	    data = mad->stream.this_frame;
-	  
+
 	    /* mad has moved the pointer to the next frame over the start of the
-	     * id3 tags, so we need to flush one byte less n the tagsize */
-	    mad_stream_skip(&mad->stream, tagsize-1);
+	     * id3 tags, so we need to flush one byte less than the tagsize */
+	    mad_stream_skip (&mad->stream, tagsize - 1);
 
 	    tag = id3_tag_parse (data, tagsize);
 	    if (tag) {
@@ -1125,7 +1146,7 @@ gst_mad_chain (GstPad *pad, GstBuffer *buffer)
 	mad->rate = rate;
       }
 
-      if (GST_PAD_IS_CONNECTED (mad->srcpad)) {
+      if (GST_PAD_IS_USABLE (mad->srcpad)) {
         GstBuffer *outbuffer;
         gint16 *outdata;
         mad_fixed_t const *left_ch, *right_ch;
