@@ -48,10 +48,9 @@ enum {
 
 enum {
   ARG_0,
-  ARG_BIT_RATE,
-  ARG_MPEG2,
   ARG_SYNC,
   ARG_MAX_DISCONT,
+  ARG_STREAMINFO,
   /* FILL ME */
 };
 
@@ -99,8 +98,10 @@ static void 		gst_mpeg_parse_get_property	(GObject *object, guint prop_id,
 							 GValue *value, GParamSpec *pspec);
 static void 		gst_mpeg_parse_set_property	(GObject *object, guint prop_id, 
 							 const GValue *value, GParamSpec *pspec);
-static void 		gst_mpeg_parse_set_cache 	(GstElement *element, GstCache *tc);
-static GstCache*	gst_mpeg_parse_get_cache 	(GstElement *element);
+static void 		gst_mpeg_parse_set_index 	(GstElement *element, GstIndex *index);
+static GstIndex*	gst_mpeg_parse_get_index 	(GstElement *element);
+
+static gboolean 	gst_mpeg_parse_release_locks 	(GstElement *element);
 
 static GstElementClass *parent_class = NULL;
 /*static guint gst_mpeg_parse_signals[LAST_SIGNAL] = { 0 };*/
@@ -137,28 +138,25 @@ gst_mpeg_parse_class_init (GstMPEGParseClass *klass)
 
   parent_class = g_type_class_ref(GST_TYPE_ELEMENT);
 
-  g_object_class_install_property(G_OBJECT_CLASS(klass), ARG_BIT_RATE,
-    g_param_spec_uint("bitrate","bitrate","bitrate",
-                      0, G_MAXUINT, 0, G_PARAM_READABLE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_MPEG2,
-    g_param_spec_boolean ("mpeg2", "mpeg2", "is this an mpeg2 stream",
-                          FALSE, G_PARAM_READABLE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_SYNC,
     g_param_spec_boolean ("sync", "Sync", "Synchronize on the stream SCR",
                           FALSE, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_MAX_DISCONT,
     g_param_spec_int ("max_discont", "Max Discont", "The maximun allowed SCR discontinuity",
                       0, G_MAXINT, DEFAULT_MAX_DISCONT, G_PARAM_READWRITE));
+  g_object_class_install_property (gobject_class, ARG_STREAMINFO,
+    g_param_spec_boxed ("streaminfo", "Streaminfo", "Streaminfo",
+                        GST_TYPE_CAPS, G_PARAM_READABLE));
 
   gobject_class->get_property = gst_mpeg_parse_get_property;
   gobject_class->set_property = gst_mpeg_parse_set_property;
 
-  gstelement_class->change_state = gst_mpeg_parse_change_state;
-  gstelement_class->get_clock    = gst_mpeg_parse_get_clock;
-  gstelement_class->set_clock    = gst_mpeg_parse_set_clock;
-  gstelement_class->get_cache    = gst_mpeg_parse_get_cache;
-  gstelement_class->set_cache    = gst_mpeg_parse_set_cache;
-
+  gstelement_class->change_state  = gst_mpeg_parse_change_state;
+  gstelement_class->get_clock     = gst_mpeg_parse_get_clock;
+  gstelement_class->set_clock     = gst_mpeg_parse_set_clock;
+  gstelement_class->get_index     = gst_mpeg_parse_get_index;
+  gstelement_class->set_index     = gst_mpeg_parse_set_index;
+  gstelement_class->release_locks = gst_mpeg_parse_release_locks;
 
   klass->parse_packhead = gst_mpeg_parse_parse_packhead;
   klass->parse_syshead 	= NULL;
@@ -174,11 +172,14 @@ gst_mpeg_parse_init (GstMPEGParse *mpeg_parse)
   mpeg_parse->sinkpad = gst_pad_new_from_template(
 		  GST_PAD_TEMPLATE_GET (sink_factory), "sink");
   gst_element_add_pad(GST_ELEMENT(mpeg_parse),mpeg_parse->sinkpad);
+  gst_pad_set_formats_function (mpeg_parse->sinkpad, gst_mpeg_parse_get_src_formats);
+  gst_pad_set_convert_function (mpeg_parse->sinkpad, gst_mpeg_parse_convert_src);
 
   mpeg_parse->srcpad = gst_pad_new_from_template(
 		  GST_PAD_TEMPLATE_GET (src_factory), "src");
   gst_element_add_pad(GST_ELEMENT(mpeg_parse),mpeg_parse->srcpad);
   gst_pad_set_formats_function (mpeg_parse->srcpad, gst_mpeg_parse_get_src_formats);
+  gst_pad_set_convert_function (mpeg_parse->srcpad, gst_mpeg_parse_convert_src);
   gst_pad_set_event_mask_function (mpeg_parse->srcpad, gst_mpeg_parse_get_src_event_masks);
   gst_pad_set_event_function (mpeg_parse->srcpad, gst_mpeg_parse_handle_src_event);
   gst_pad_set_query_type_function (mpeg_parse->srcpad, gst_mpeg_parse_get_src_query_types);
@@ -186,20 +187,13 @@ gst_mpeg_parse_init (GstMPEGParse *mpeg_parse)
 
   gst_element_set_loop_function (GST_ELEMENT (mpeg_parse), gst_mpeg_parse_loop);
 
-  /* initialize parser state */
   mpeg_parse->packetize = NULL;
-  mpeg_parse->current_scr = 0;
-  mpeg_parse->bytes_since_scr = 0;
-  mpeg_parse->adjust = 0;
   mpeg_parse->sync = FALSE;
-
-  /* zero counters (should be done at RUNNING?) */
-  mpeg_parse->mux_rate = 0;
-  mpeg_parse->discont_pending = FALSE;
-  mpeg_parse->scr_pending = TRUE;
+  mpeg_parse->id = NULL;
   mpeg_parse->max_discont = DEFAULT_MAX_DISCONT;
   mpeg_parse->provided_clock = gst_mpeg_clock_new ("MPEGParseClock", 
 		  gst_mpeg_parse_get_time, mpeg_parse);
+  mpeg_parse->streaminfo = NULL;
 
   GST_FLAG_SET (mpeg_parse, GST_ELEMENT_EVENT_AWARE);
 }
@@ -207,9 +201,9 @@ gst_mpeg_parse_init (GstMPEGParse *mpeg_parse)
 static GstClock*
 gst_mpeg_parse_get_clock (GstElement *element)
 {   
-  //GstMPEGParse *parse = GST_MPEG_PARSE (element);
+  /* GstMPEGParse *parse = GST_MPEG_PARSE (element); */
 
-  //return parse->provided_clock;
+  /* return parse->provided_clock; */
   return NULL;
 }
 
@@ -229,6 +223,31 @@ gst_mpeg_parse_get_time (GstClock *clock, gpointer data)
   return MPEGTIME_TO_GSTTIME (parse->current_scr);
 }
 
+static void
+gst_mpeg_parse_update_streaminfo (GstMPEGParse *mpeg_parse)
+{
+  GstProps *props;
+  GstPropsEntry *entry;
+  gboolean mpeg2 = GST_MPEG_PACKETIZE_IS_MPEG2 (mpeg_parse->packetize);
+
+  props = gst_props_empty_new ();
+
+  entry = gst_props_entry_new ("mpegversion", GST_PROPS_INT (mpeg2 ? 2 : 1));
+  gst_props_add_entry (props, (GstPropsEntry *) entry);
+
+  entry = gst_props_entry_new ("bitrate", GST_PROPS_INT (mpeg_parse->mux_rate * 400)); 
+  gst_props_add_entry (props, (GstPropsEntry *) entry);
+
+  if (mpeg_parse->streaminfo) 
+    gst_caps_unref (mpeg_parse->streaminfo);
+
+  mpeg_parse->streaminfo = gst_caps_new ("mpeg_streaminfo",
+		                         "application/x-gst-streaminfo",
+					  props);
+
+  g_object_notify (G_OBJECT (mpeg_parse), "streaminfo");
+}
+
 static void
 gst_mpeg_parse_send_data (GstMPEGParse *mpeg_parse, GstData *data, GstClockTime time)
 {
@@ -245,14 +264,18 @@ gst_mpeg_parse_send_data (GstMPEGParse *mpeg_parse, GstData *data, GstClockTime
     if (!GST_PAD_CAPS (mpeg_parse->srcpad)) {
       gboolean mpeg2 = GST_MPEG_PACKETIZE_IS_MPEG2 (mpeg_parse->packetize);
 
-      gst_pad_try_set_caps (mpeg_parse->srcpad,
+      if (gst_pad_try_set_caps (mpeg_parse->srcpad,
 		      GST_CAPS_NEW (
     			"mpeg_parse_src",
     			"video/mpeg",
     			  "mpegversion",  GST_PROPS_INT (mpeg2 ? 2 : 1),
     			  "systemstream", GST_PROPS_BOOLEAN (TRUE),
     			  "parsed",       GST_PROPS_BOOLEAN (TRUE)
-			      ));
+			      )) < 0)
+      {
+	gst_element_error (GST_ELEMENT (mpeg_parse), "could no set source caps");
+	return;
+      }
     }
 
     GST_BUFFER_TIMESTAMP (data) = time;
@@ -329,6 +352,10 @@ gst_mpeg_parse_parse_packhead (GstMPEGParse *mpeg_parse, GstBuffer *buffer)
   mpeg_parse->bytes_since_scr = 0;
   scr_adj = scr + mpeg_parse->adjust;
 
+  if (mpeg_parse->next_scr == -1) {
+    mpeg_parse->next_scr = scr;
+  }
+
   GST_DEBUG (0, "SCR is %llu (%llu) next: %lld (%lld) diff: %lld (%lld)", 
 		  scr, 
 		  MPEGTIME_TO_GSTTIME (scr),
@@ -351,13 +378,12 @@ gst_mpeg_parse_parse_packhead (GstMPEGParse *mpeg_parse, GstBuffer *buffer)
     scr = scr_adj;
   }
 
-  if (mpeg_parse->cache) {
-    /* update cache if any */
-    gst_cache_add_association (mpeg_parse->cache, mpeg_parse->cache_id, 
-		    	       GST_ACCOCIATION_FLAG_KEY_UNIT,
+  if (mpeg_parse->index && GST_INDEX_IS_WRITABLE (mpeg_parse->index)) {
+    /* update index if any */
+    gst_index_add_association (mpeg_parse->index, mpeg_parse->index_id, 
+		    	       GST_ASSOCIATION_FLAG_KEY_UNIT,
 		               GST_FORMAT_BYTES, GST_BUFFER_OFFSET (buffer), 
-			       GST_FORMAT_TIME, MPEGTIME_TO_GSTTIME (scr), 
-			       scr_format, scr_orig, 0);
+			       GST_FORMAT_TIME, MPEGTIME_TO_GSTTIME (scr), 0);
   }
 
   mpeg_parse->current_scr = scr;
@@ -366,11 +392,10 @@ gst_mpeg_parse_parse_packhead (GstMPEGParse *mpeg_parse, GstBuffer *buffer)
   if (mpeg_parse->mux_rate != new_rate) {
     mpeg_parse->mux_rate = new_rate;
 
-    g_object_notify (G_OBJECT (mpeg_parse), "bitrate");
+    gst_mpeg_parse_update_streaminfo (mpeg_parse);
+    GST_DEBUG (0, "stream is %1.3fMbs", (mpeg_parse->mux_rate * 400) / 1000000.0);
   }
 
-  GST_DEBUG (0, "stream is %1.3fMbs", (mpeg_parse->mux_rate * 400) / 1000000.0);
-
   return TRUE;
 }
 
@@ -433,7 +458,6 @@ gst_mpeg_parse_loop (GstElement *element)
         GST_DEBUG (GST_CAT_EVENT, "event: %d\n", GST_EVENT_TYPE (data));
 
         mpeg_parse->discont_pending = TRUE;
-        mpeg_parse->scr_pending = TRUE;
         mpeg_parse->packetize->resync = TRUE;
 	gst_event_unref (event);
 	return;
@@ -470,14 +494,36 @@ gst_mpeg_parse_loop (GstElement *element)
     size = GST_BUFFER_SIZE (data);
     mpeg_parse->bytes_since_scr += size;
 
+    if (!GST_PAD_CAPS (mpeg_parse->sinkpad)) {
+      gboolean mpeg2 = GST_MPEG_PACKETIZE_IS_MPEG2 (mpeg_parse->packetize);
+
+      if (gst_pad_try_set_caps (mpeg_parse->sinkpad,
+		      GST_CAPS_NEW (
+    			"mpeg_parse_src",
+    			"video/mpeg",
+    			  "mpegversion",  GST_PROPS_INT (mpeg2 ? 2 : 1),
+    			  "systemstream", GST_PROPS_BOOLEAN (TRUE),
+    			  "parsed",       GST_PROPS_BOOLEAN (TRUE)
+			      )) < 0)
+      {
+	gst_element_error (GST_ELEMENT (mpeg_parse), "could no set sink caps");
+	return;
+      }
+    }
+
     if (CLASS (mpeg_parse)->send_data)
       CLASS (mpeg_parse)->send_data (mpeg_parse, data, time);
 
     if (mpeg_parse->clock && mpeg_parse->sync && !mpeg_parse->discont_pending) {
+      mpeg_parse->id = gst_clock_new_single_shot_id (mpeg_parse->clock, time);
+
       GST_DEBUG (GST_CAT_CLOCK, "syncing mpegparse");
-      gst_element_clock_wait (GST_ELEMENT (mpeg_parse), mpeg_parse->clock, time, NULL);
+      gst_element_clock_wait (GST_ELEMENT (mpeg_parse), mpeg_parse->id, NULL);
+      gst_clock_id_free (mpeg_parse->id);
+      mpeg_parse->id = NULL;
     }
 
+    if (mpeg_parse->current_scr != -1)
     {
       guint64 scr, bss, br;
 
@@ -514,73 +560,100 @@ const GstFormat*
 gst_mpeg_parse_get_src_formats (GstPad *pad)
 {
   static const GstFormat formats[] = {
-    GST_FORMAT_TIME,
+    GST_FORMAT_BYTES,
+    GST_FORMAT_TIME,		
     0 
   };
   return formats;
 }
 
-const GstPadQueryType*
+gboolean
+gst_mpeg_parse_convert_src (GstPad *pad, GstFormat src_format, gint64 src_value,
+		            GstFormat *dest_format, gint64 *dest_value)
+{
+  gboolean res = TRUE;
+  GstMPEGParse *mpeg_parse = GST_MPEG_PARSE (gst_pad_get_parent (pad));
+
+  switch (src_format) {
+    case GST_FORMAT_BYTES:
+      switch (*dest_format) {
+        case GST_FORMAT_DEFAULT:
+          *dest_format = GST_FORMAT_TIME;
+        case GST_FORMAT_TIME:
+	  if (mpeg_parse->mux_rate == 0)
+	    res = FALSE;
+	  else
+            *dest_value = src_value * GST_SECOND / (mpeg_parse->mux_rate * 50);
+          break;
+        default:
+          res = FALSE;
+      }
+      break;
+    case GST_FORMAT_TIME:
+      switch (*dest_format) {
+        case GST_FORMAT_DEFAULT:
+          *dest_format = GST_FORMAT_BYTES;
+        case GST_FORMAT_BYTES:
+          *dest_value = mpeg_parse->mux_rate * 50 * src_value / GST_SECOND;
+          break;
+        default:
+          res = FALSE;
+      }
+      break;
+    default:
+      res = FALSE;
+      break;
+  }
+  return res;
+}
+
+const GstQueryType*
 gst_mpeg_parse_get_src_query_types (GstPad *pad)
 {
-  static const GstPadQueryType types[] = {
-    GST_PAD_QUERY_TOTAL,
-    GST_PAD_QUERY_POSITION,
+  static const GstQueryType types[] = {
+    GST_QUERY_TOTAL,
+    GST_QUERY_POSITION,
     0 
   };
   return types;
 }
 
 gboolean
-gst_mpeg_parse_handle_src_query (GstPad *pad, GstPadQueryType type, 
+gst_mpeg_parse_handle_src_query (GstPad *pad, GstQueryType type, 
 			         GstFormat *format, gint64 *value)
 {
   gboolean res = TRUE;
   GstMPEGParse *mpeg_parse = GST_MPEG_PARSE (gst_pad_get_parent (pad));
+  GstFormat src_format;
+  gint64 src_value;
 
   switch (type) {
-    case GST_PAD_QUERY_TOTAL:
+    case GST_QUERY_TOTAL:
     {
       switch (*format) {
         case GST_FORMAT_DEFAULT:
           *format = GST_FORMAT_TIME;
           /* fallthrough */
-        case GST_FORMAT_TIME:
-	{
-          GstFormat peer_format;
-	  gint64 peer_value;
-
-	  if (mpeg_parse->mux_rate == 0) 
-	    return FALSE;
-
-	  peer_format = GST_FORMAT_BYTES;
-	  if (gst_pad_query (GST_PAD_PEER (mpeg_parse->sinkpad),
-			     GST_PAD_QUERY_TOTAL, &peer_format, &peer_value)) 
-	  {
-            /* multiply bywith 8 because vbr is in bits/second */
-            *value = peer_value * GST_SECOND / (mpeg_parse->mux_rate * 50);
-          }
-	  else 
-	    res = FALSE;
-	  break;
-	}
 	default:
-	  res = FALSE;
+	  src_format = GST_FORMAT_BYTES;
+	  if (!gst_pad_query (GST_PAD_PEER (mpeg_parse->sinkpad),
+			      GST_QUERY_TOTAL, &src_format, &src_value)) 
+	  {
+	    res = FALSE;
+	  }
 	  break;
       }
       break;
     }
-    case GST_PAD_QUERY_POSITION:
+    case GST_QUERY_POSITION:
     {
       switch (*format) {
         case GST_FORMAT_DEFAULT:
           *format = GST_FORMAT_TIME;
           /* fallthrough */
-        case GST_FORMAT_TIME:
-          *value = MPEGTIME_TO_GSTTIME (mpeg_parse->current_scr);
-	  break;
 	default:
-	  res = FALSE;
+	  src_format = GST_FORMAT_TIME;
+          src_value = MPEGTIME_TO_GSTTIME (mpeg_parse->current_scr);
 	  break;
       }
       break;
@@ -590,6 +663,10 @@ gst_mpeg_parse_handle_src_query (GstPad *pad, GstPadQueryType type,
       break;
   }
 
+  /* bring to requested format */
+  if (res)
+    res = gst_pad_convert (pad, src_format, src_value, format, value);
+
   return res;
 }
 
@@ -603,39 +680,94 @@ gst_mpeg_parse_get_src_event_masks (GstPad *pad)
   return masks;
 }
 
+static gboolean
+index_seek (GstPad *pad, GstEvent *event, guint64 *offset, gint64 *scr)
+{
+  GstIndexEntry *entry;
+  GstMPEGParse *mpeg_parse = GST_MPEG_PARSE (gst_pad_get_parent (pad));
+
+  entry = gst_index_get_assoc_entry (mpeg_parse->index, mpeg_parse->index_id,
+                                     GST_INDEX_LOOKUP_BEFORE, 0,
+                                     GST_EVENT_SEEK_FORMAT (event),
+	                             GST_EVENT_SEEK_OFFSET (event));
+  if (!entry)
+    return FALSE;
+
+  if (gst_index_entry_assoc_map (entry, GST_FORMAT_BYTES, offset)) {
+    gint64 time;
+
+    if (gst_index_entry_assoc_map (entry, GST_FORMAT_TIME, &time)) {
+      *scr = GSTTIME_TO_MPEGTIME (time);
+    }
+    return TRUE;
+  }
+  return FALSE;
+}
+
+static gboolean
+normal_seek (GstPad *pad, GstEvent *event, guint64 *offset, gint64 *scr)
+{
+  gboolean res;
+  GstFormat format;
+  gint64 time;
+
+  /* bring offset to bytes */
+  format = GST_FORMAT_BYTES;
+  res  = gst_pad_convert (pad, 
+		          GST_EVENT_SEEK_FORMAT (event),
+		          GST_EVENT_SEEK_OFFSET (event),
+		          &format,
+		          offset);
+  /* bring offset to time */
+  format = GST_FORMAT_TIME;
+  res &= gst_pad_convert (pad, 
+		          GST_EVENT_SEEK_FORMAT (event),
+		          GST_EVENT_SEEK_OFFSET (event),
+			  &format,
+			  &time);
+
+  /* convert to scr */
+  *scr = GSTTIME_TO_MPEGTIME (time);
+
+  return res;
+}
+
 gboolean
 gst_mpeg_parse_handle_src_event (GstPad *pad, GstEvent *event)
 {
-  gboolean res = TRUE;
+  gboolean res = FALSE;
   GstMPEGParse *mpeg_parse = GST_MPEG_PARSE (gst_pad_get_parent (pad));
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_SEEK:
     {
       guint64 desired_offset;
+      guint64 expected_scr;
 
-      if (GST_EVENT_SEEK_FORMAT (event) != GST_FORMAT_TIME) {
-        gst_event_unref (event);
-        return FALSE;
-      }
+      /* first to to use the index if we have one */
+      if (mpeg_parse->index) 
+        res = index_seek (pad, event, &desired_offset, &expected_scr);
+      /* nothing found, try fuzzy seek */
+      if (!res) 
+        res = normal_seek (pad, event, &desired_offset, &expected_scr);
 
-      if (GST_EVENT_SEEK_FLAGS (event) & GST_SEEK_FLAG_FLUSH) {
+      if (!res)
+	break;
+	      
+      GST_DEBUG (0, "sending seek to %lld", desired_offset);
+      if (gst_bytestream_seek (mpeg_parse->packetize->bs, desired_offset, GST_SEEK_METHOD_SET)) {
+        mpeg_parse->discont_pending = TRUE;
+        mpeg_parse->scr_pending = TRUE;
+        mpeg_parse->next_scr = expected_scr;
+        mpeg_parse->current_scr = -1;
+        mpeg_parse->adjust = 0;
+        res = TRUE;
       }
-      desired_offset = mpeg_parse->mux_rate * 50 * GST_EVENT_SEEK_OFFSET (event) / (GST_SECOND);
-
-      if (!gst_bytestream_seek (mpeg_parse->packetize->bs, desired_offset, GST_SEEK_METHOD_SET)) {
-        gst_event_unref (event);
-	return FALSE;
-      }
-      mpeg_parse->discont_pending = TRUE;
-      mpeg_parse->scr_pending = TRUE;
       break;
     }
     default:
-      res = FALSE;
       break;
   }
-
   gst_event_unref (event);
   return res;
 }
@@ -650,6 +782,16 @@ gst_mpeg_parse_change_state (GstElement *element)
       if (!mpeg_parse->packetize) {
         mpeg_parse->packetize = gst_mpeg_packetize_new (mpeg_parse->sinkpad, GST_MPEG_PACKETIZE_SYSTEM);
       }
+      /* initialize parser state */
+      mpeg_parse->current_scr = 0;
+      mpeg_parse->bytes_since_scr = 0;
+      mpeg_parse->adjust = 0;
+      mpeg_parse->next_scr = 0;
+
+      /* zero counters (should be done at RUNNING?) */
+      mpeg_parse->mux_rate = 0;
+      mpeg_parse->discont_pending = FALSE;
+      mpeg_parse->scr_pending = FALSE;
       break;
     case GST_STATE_PAUSED_TO_READY:
       if (mpeg_parse->packetize) {
@@ -669,25 +811,18 @@ gst_mpeg_parse_get_property (GObject *object, guint prop_id, GValue *value, GPar
 {
   GstMPEGParse *mpeg_parse;
 
-  /* it's not null if we got it, but it might not be ours */
   mpeg_parse = GST_MPEG_PARSE(object);
 
   switch (prop_id) {
-    case ARG_BIT_RATE: 
-      g_value_set_uint (value, mpeg_parse->mux_rate * 400); 
-      break;
-    case ARG_MPEG2:
-      if (mpeg_parse->packetize)
-        g_value_set_boolean (value, GST_MPEG_PACKETIZE_IS_MPEG2 (mpeg_parse->packetize));
-      else
-        g_value_set_boolean (value, FALSE);
-      break;
     case ARG_SYNC: 
       g_value_set_boolean (value, mpeg_parse->sync);
       break;
     case ARG_MAX_DISCONT: 
       g_value_set_int (value, mpeg_parse->max_discont);
       break;
+    case ARG_STREAMINFO: 
+      g_value_set_boxed (value, mpeg_parse->streaminfo);
+      break;
     default: 
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -716,27 +851,41 @@ gst_mpeg_parse_set_property (GObject *object, guint prop_id,
 }
 
 static void
-gst_mpeg_parse_set_cache (GstElement *element, GstCache *cache)
+gst_mpeg_parse_set_index (GstElement *element, GstIndex *index)
 {
   GstMPEGParse *mpeg_parse;
 
   mpeg_parse = GST_MPEG_PARSE (element);
   
-  mpeg_parse->cache = cache;
+  mpeg_parse->index = index;
 
-  gst_cache_get_writer_id (cache, GST_OBJECT (mpeg_parse->sinkpad),
-		           &mpeg_parse->cache_id);
-  gst_cache_add_format (cache, mpeg_parse->cache_id, scr_format);
+  gst_index_get_writer_id (index, GST_OBJECT (mpeg_parse->sinkpad),
+		           &mpeg_parse->index_id);
+  gst_index_add_format (index, mpeg_parse->index_id, scr_format);
 }
 
-static GstCache*
-gst_mpeg_parse_get_cache (GstElement *element)
+static GstIndex*
+gst_mpeg_parse_get_index (GstElement *element)
 {
   GstMPEGParse *mpeg_parse;
 
   mpeg_parse = GST_MPEG_PARSE (element);
   
-  return mpeg_parse->cache;
+  return mpeg_parse->index;
+}
+
+static gboolean
+gst_mpeg_parse_release_locks (GstElement *element)
+{
+  GstMPEGParse *mpeg_parse;
+
+  mpeg_parse = GST_MPEG_PARSE (element);
+
+  if (mpeg_parse->id) {
+    gst_clock_id_unlock (mpeg_parse->id);
+  }
+
+  return TRUE;
 }
 
 gboolean
