@@ -119,8 +119,12 @@ static void			gst_videosink_release_conn	(GstVideoSink *sink);
 static void			gst_videosink_append_cache	(GstVideoSink *sink, GstImageData *image);
 static gboolean			gst_videosink_set_caps		(GstVideoSink *sink, GstCaps *caps);
 /* bufferpool stuff */
-static GstBuffer*		gst_videosink_buffer_new 	(GstBufferPool *pool, guint64 location, guint size, gpointer user_data);
-static void			gst_videosink_buffer_free	(GstBufferPool *pool, GstBuffer *buffer, gpointer user_data);
+static GstBuffer*		gst_videosink_buffer_new 	(GstBufferPool *pool, 
+								 gint64 location, 
+								 guint size, gpointer user_data);
+static void			gst_videosink_buffer_free	(GstBufferPool *pool, 
+		                                                 GstBuffer *buffer, 
+								 gpointer user_data);
 
 /* prototypes from plugins */
 extern GstImagePlugin* 		get_ximage_plugin		(void);
@@ -224,9 +228,9 @@ gst_videosink_init (GstVideoSink *sink)
   sink->bufferpool = gst_buffer_pool_new (
 		    NULL,
 		    NULL,
-		    gst_videosink_buffer_new,
+		    (GstBufferPoolBufferNewFunction)gst_videosink_buffer_new,
 		    NULL,
-		    gst_videosink_buffer_free,
+		    (GstBufferPoolBufferFreeFunction)gst_videosink_buffer_free,
 		    sink);
   sink->cache_lock = g_mutex_new();
   sink->cache = NULL; 
@@ -270,7 +274,8 @@ gst_videosink_append_cache (GstVideoSink *sink, GstImageData *image)
   g_mutex_unlock (sink->cache_lock);
 }
 static GstBuffer*
-gst_videosink_buffer_new (GstBufferPool *pool, guint64 location, guint size, gpointer user_data)
+gst_videosink_buffer_new (GstBufferPool *pool, gint64 location, 
+		          guint size, gpointer user_data)
 {
   GstVideoSink *sink;
   GstBuffer *buffer;
@@ -419,8 +424,10 @@ gst_videosink_chain (GstPad *pad, GstBuffer *buf)
 
   if (sink->clock && time != -1) {
     GstClockReturn ret;
+    GstClockID id = gst_clock_new_single_shot_id (sink->clock, GST_BUFFER_TIMESTAMP (buf));
 
-    ret = gst_element_clock_wait (GST_ELEMENT (sink), sink->clock, GST_BUFFER_TIMESTAMP (buf), NULL);
+    ret = gst_element_clock_wait (GST_ELEMENT (sink), id, NULL);
+    gst_clock_id_free (id);
 
     /* we are going to drop early buffers */
     if (ret == GST_CLOCK_EARLY) {
@@ -442,10 +449,14 @@ gst_videosink_chain (GstPad *pad, GstBuffer *buf)
       sink->plugin->put_image (sink->hook, (GstImageData *) GST_BUFFER_POOL_PRIVATE (buf));
       sink->last_image = buf;
     } else {
-      buffer = gst_buffer_new_from_pool (gst_videosink_get_bufferpool (sink->sinkpad), 0, GST_BUFFER_SIZE (buf));
-      memcpy (GST_BUFFER_DATA (buffer), GST_BUFFER_DATA (buf), GST_BUFFER_SIZE (buf) > GST_BUFFER_SIZE (buffer) ? 
-	      GST_BUFFER_SIZE (buffer) : GST_BUFFER_SIZE (buf));
+      buffer = gst_buffer_new_from_pool (gst_videosink_get_bufferpool (sink->sinkpad), 
+		                         0, GST_BUFFER_SIZE (buf));
+      memcpy (GST_BUFFER_DATA (buffer), GST_BUFFER_DATA (buf), 
+	      GST_BUFFER_SIZE (buf) > GST_BUFFER_SIZE (buffer) ? 
+	        GST_BUFFER_SIZE (buffer) : GST_BUFFER_SIZE (buf));
+
       sink->plugin->put_image (sink->hook, (GstImageData *) GST_BUFFER_POOL_PRIVATE (buffer));
+
       sink->last_image = buffer;
       gst_buffer_unref (buf);
     }
